const { get } = require('snekfetch'); // eslint-disable-line
// Blame eslint

module.exports = class Clash {

    constructor(key = process.env.COC_API) {
        if (!key) throw new TypeError('You must provide a key while initializing this class or set a key in your environment variables.');

        /**
        * The API key used for authentication
        * @since 0.0.1
        */
        this.key = key;

        /**
        * The base URL for all API endpoints
        * @since 0.0.1
        */
        this.base_URL = 'https://api.clashofclans.com/v1/';
    }

    /**
    * @typedef {object} Player
    * @property {string} tag The tag of the player
    * @property {string} name The name of the player
    * @property {number} expLevel The experience level of the player
    * @property {object} league The league the player is in
    * @property {number} trophies The number of trophies the player has
    * @property {number} versusTrophies The number of versus trophies the player has
    * @property {number} attackWins The number of attacks the player has won
    * @property {number} defenseWins The number of defenses the player has won
    */

    /**
     * Returns a player object
     * @since 0.0.1
     * @param {string} tag The player tag of the player
     * @returns {Promise<Player>}
     */
    async getPlayer(tag) {
        const _tag = Clash.resolveTag(tag);
        return this._get(`players/${_tag}`);
    }

    /**
    * @typedef {object} Clan
    * @property {string} name The name of the clan
    * @property {string} tag The tag of the clan
    * @property {object} location The location of the clan with additional properties
    * @property {object} badgeUrls The badge urls of the clan in differemt sizes
    * @property {number} clanLevel The level of the clan
    * @property {number} clanPoints The average trophies of the clan
    * @property {number} clanVersusPoints The average versus battle trophies of the clan
    * @property {number} members The number of members of this clan
    * @property {string} type The type of this clan (invite only/closed/open/etc)
    * @property {number} requiredTrophies The amount of trophies required to join
    * @property {string} warFrequency The war frequency of the clan
    * @property {number} warWins The number of clan wars the clan has won
    * @property {number} warLosses The number of clan wars rhis clan has lost
    * @property {number} warTies The number of clan wars this clan has tied
    * @property {boolean} isWarLogPublic Whether the clan's war log is public
    * @property {string} description The description of the clan
    */

    /**
    * Returns a clan object from the API
    * @since 0.0.1
    * @param {string} tag The tag of the clan
    * @returns {Promise<Clan>}
    */
    async getClan(tag) {
        const _tag = Clash.resolveTag(tag);
        const body = this._get(`clans/${_tag}`);
        return body.items;
    }

    /**
    * @typedef {object} ClanWarLog
    * @property {string} result The result of the war
    * @property {string} endTime The time the war ends/ended
    * @property {number} teamSize The number of members in the war
    * @property {Clan} clan The parent clan of this war log
    * @property {Clan} opponent The opposing clan
    */

    /**
    * Returns a war log of a clan
    * @since 0.1.0
    * @param {string} tag The tag of the clan
    * @returns {Promise<Array<ClanWarLog>>}
    */
    async getWarLog(tag) {
        const _tag = Clash.resolveTag(tag);
        return this._get(`clans/${_tag}/warlog`);
    }

    /**
     * Returns the current war of the clan
     * @since 0.1.0
     * @param {string} tag The tag of the clan
     * @returns {ClanWarLog}
     */
    async currentWar(tag) {
        const _tag = Clash.resolveTag(tag);
        return this._get(`clans/${_tag}/currentwar`);
    }

    /**
    * Used for making GET requests to the API
    * @since 0.1.0
    * @private
    * @param {string} endpoint The endpoint to make the GET request to
    * @returns {Promise<*>}
    */
    async _get(endpoint) {
        const { body } = await get(`${this.base_URL}${endpoint}`)
            .set('Authorization', `Bearer ${this.key}`)
            .catch(err => Promise.reject(err));
        return body || false;
    }

    /**
     * Resolves a Clash of Clans tag and returns an encoded tag
     * @static
     * @since 0.1.0
     * @param {string} tag The clan tag to resolve
     * @returns {string}
     */
    static resolveTag(tag) {
        if (!tag) throw new TypeError('Invalid arguments: this function expects a clan tag.');
        if (typeof tag !== 'string') throw new TypeError('Clan tag must be a string.');
        tag = tag.toUpperCase();
        const _tag = /[#][0289PYLQGRJCUV]{4,9}/i.test(tag);
        if (!_tag) throw new TypeError('The provided tag was in a incorrect format!');
        return encodeURIComponent(_tag);
    }

};
